@@ -5,8 +5,10 @@
 #' @param blocks A vector of length N that indicates which block each unit belongs to.
 #' @param clusters A vector of length N that indicates which cluster each unit belongs to.
 #' @param prob Use for a two-arm design in which either floor(N_clusters_block*prob) or ceiling(N_clusters_block*prob) clusters are assigned to treatment within each block. The probability of assignment to treatment is exactly prob because with probability 1-prob, floor(N_clusters_block*prob) clusters will be assigned to treatment and with probability prob, ceiling(N_clusters_block*prob) clusters will be assigned to treatment. prob must be a real number between 0 and 1 inclusive. (optional)
+#' @param prob_unit Use for a two arm design. Must of be of length N. tapply(prob_unit, blocks, unique) will be passed to \code{block_prob}.
 #' @param prob_each Use for a multi-arm design in which the values of prob_each determine the probabilities of assignment to each treatment condition. prob_each must be a numeric vector giving the probability of assignment to each condition. All entries must be nonnegative real numbers between 0 and 1 inclusive and the total must sum to 1. Because of integer issues, the exact number of clusters assigned to each condition may differ (slightly) from assignment to assignment, but the overall probability of assignment is exactly prob_each. (optional)
 #' @param m Use for a two-arm design in which the scalar m describes the fixed number of clusters assigned in each block. This number does not vary across blocks.
+#' @param m_unit Use for a two-arm design. Must be of length N. tapply(m_unit, blocks, unique) will be passed to \code{block_m}.
 #' @param block_m Use for a two-arm design in which block_m describes the number of clusters to assign to treatment within each block. block_m must be a numeric vector that is as long as the number of blocks and is in the same order as sort(unique(blocks)).
 #' @param block_m_each Use for a multi-arm design in which the values of block_m_each determine the number of clusters assigned to each condition. block_m_each must be a matrix with the same number of rows as blocks and the same number of columns as treatment arms. Cell entries are the number of clusters to be assigned to each treatment arm within each block. The rows should respect the ordering of the blocks as determined by sort(unique(blocks)). The columns should be in the order of conditions, if specified.
 #' @param block_prob Use for a two-arm design in which block_prob describes the probability of assignment to treatment within each block. Must be in the same order as sort(unique(blocks)). Differs from prob in that the probability of assignment can vary across blocks. 
@@ -65,8 +67,10 @@ block_and_cluster_ra <-
   function(blocks = NULL,
            clusters = NULL,
            prob = NULL,
+           prob_unit = NULL,
            prob_each = NULL,
            m = NULL,
+           m_unit = NULL,
            block_m = NULL,
            block_m_each = NULL,
            block_prob = NULL,
@@ -83,6 +87,14 @@ block_and_cluster_ra <-
 
     # get the block for each cluster
     clust_blocks <- tapply(blocks, clusters, unique)
+    
+    if(!is.null(prob_unit)){
+      block_prob <- tapply(prob_unit, blocks, unique)
+    }
+    
+    if(!is.null(m_unit)){
+      block_m <- tapply(m_unit, blocks, unique)
+    }
 
     # Conduct random assignment at cluster level
     z_clust <- block_ra(
@@ -155,8 +167,10 @@ block_and_cluster_ra_probabilities <-
   function(blocks = NULL,
            clusters = NULL,
            prob = NULL,
+           prob_unit = NULL,
            prob_each = NULL,
            m = NULL,
+           m_unit = NULL,
            block_m = NULL,
            block_m_each = NULL,
            block_prob = NULL,
@@ -176,6 +190,14 @@ block_and_cluster_ra_probabilities <-
     # get the block for each cluster
     clust_blocks <- tapply(blocks, clusters, unique)
     
+    if(!is.null(prob_unit)){
+      block_prob <- tapply(prob_unit, blocks, unique)
+    }
+    
+    if(!is.null(m_unit)){
+      block_m <- tapply(m_unit, blocks, unique)
+    }
+    
     probs_clust <- block_ra_probabilities(
       blocks = clust_blocks,
       prob = prob,
